//
//  AccelerometerController.m
//  Accelerometer
//
//  Created by Phidgets on 2015-07-30.
//  Copyright (c) 2015 Phidgets. All rights reserved.
//

#import "AccelerometerController.h"

@implementation AccelerometerController

- (void)windowWillClose:(NSNotification*)aNotification {
    if(ch){
        //Safely close example
        Phidget_setOnAttachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnDetachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnErrorHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_close((PhidgetHandle)ch);
        PhidgetAccelerometer_delete(&ch);
        ch = NULL;
    }
}

-(void)awakeFromNib{
    PhidgetReturnCode result;
    //Manage GUI
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
    [mainWindow setDelegate:self];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    [phidgetInfoBox setHidden:NO];
    
    result = PhidgetAccelerometer_create(&ch);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    result = [self initChannel:(PhidgetHandle)ch];
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    result = PhidgetAccelerometer_setOnAccelerationChangeHandler(ch, gotAccelerometerData, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }

    [PhidgetInfoBox openCmdLine:(PhidgetHandle)ch];
}

-(PhidgetReturnCode)initChannel:(PhidgetHandle) channel{
    PhidgetReturnCode result;
    
    result = Phidget_setOnAttachHandler(channel, gotAttach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnDetachHandler(channel, gotDetach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnErrorHandler(channel, gotError, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    /*
     * Please review the Phidget22 channel matching documentation for details on the device
     * and class architecture of Phidget22, and how channels are matched to device features.
     */
    
    /*
     * Specifies the serial number of the device to attach to.
     * For VINT devices, this is the hub serial number.
     *
     * The default is any device.
     */
    // Phidget_setDeviceSerialNumber(ch, <YOUR DEVICE SERIAL NUMBER>);
    
    /*
     * For VINT devices, this specifies the port the VINT device must be plugged into.
     *
     * The default is any port.
     */
    // Phidget_setHubPort(ch, 0);
    
    /*
     * Specifies that the channel should only match a VINT hub port.
     * The only valid channel id is 0.
     *
     * The default is 0 (false), meaning VINT hub ports will never match
     */
    // Phidget_setIsHubPortDevice(ch, 1);
    
    /*
     * Specifies which channel to attach to.  It is important that the channel of
     * the device is the same class as the channel that is being opened.
     *
     * The default is any channel.
     */
    // Phidget_setChannel(ch, 0);
    
    /*
     * In order to attach to a network Phidget, the program must connect to a Phidget22 Network Server.
     * In a normal environment this can be done automatically by enabling server discovery, which
     * will cause the client to discovery and connect to available servers.
     *
     * To force the channel to only match a network Phidget, set remote to 1.
     */
    // PhidgetNet_enableServerDiscovery(PHIDGETSERVER_DEVICEREMOTE);
    // Phidget_setIsRemote(ch, 1);
    
    return EPHIDGET_OK;
}

#pragma mark Event callbacks
static void gotAttach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onAttachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}

static void gotDetach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onDetachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}

static void gotError(PhidgetHandle phid, void *context, Phidget_ErrorEventCode errcode, const char *error){
    [(__bridge id)context performSelectorOnMainThread:@selector(errorHandler:)
                                           withObject:[NSArray arrayWithObjects:[NSNumber numberWithInt:errcode], [NSString stringWithUTF8String:error], nil]
                                        waitUntilDone:NO];
}

void gotAccelerometerData(PhidgetAccelerometerHandle phid, void *context, const double acceleration[3], double timestamp){
    [(__bridge id)context performSelectorOnMainThread:@selector(onAccelerometerChangeHandler:)
                                           withObject:[NSArray arrayWithObjects:[NSNumber numberWithDouble:acceleration[0]],[NSNumber numberWithDouble:acceleration[1]],[NSNumber numberWithDouble:acceleration[2]], [NSNumber numberWithDouble:timestamp], nil]
                                        waitUntilDone:NO];
}

#pragma mark Attach, detach, data, and error events
- (void)onAttachHandler{
    int axisCount = 0;
    Phidget_DeviceID deviceID;
    double minAccelerationChangeTrigger,maxAccelerationChangeTrigger, accelerationChangeTrigger;
    uint32 minDataInterval,maxDataInterval, dataInterval;
    const char *deviceSKU;
    
    //Get information from channel which will allow us to configure the GUI properly
    Phidget_getDeviceID((PhidgetHandle)ch, &deviceID);
    Phidget_getDeviceSKU((PhidgetHandle)ch, &deviceSKU);
    PhidgetAccelerometer_getMinAccelerationChangeTrigger(ch, &minAccelerationChangeTrigger);
    PhidgetAccelerometer_getMaxAccelerationChangeTrigger(ch, &maxAccelerationChangeTrigger);
    PhidgetAccelerometer_getAccelerationChangeTrigger(ch, &accelerationChangeTrigger);
    PhidgetAccelerometer_getMinDataInterval(ch, &minDataInterval);
    PhidgetAccelerometer_getMaxDataInterval(ch, &maxDataInterval);
    PhidgetAccelerometer_getDataInterval(ch, &dataInterval);
    PhidgetAccelerometer_getAxisCount(ch, &axisCount);
    
    //Adjust GUI based on information from channel
    [phidgetInfoBoxView fillPhidgetInfo:(PhidgetHandle)ch];
    [settingsBox setHidden:NO];
    [dataBox setHidden:NO];
    
    //Configure example based on axis count
    if(axisCount == 3){
        [xViewBox setHidden:NO];
        [yViewBox setHidden:NO];
        [zViewBox setHidden:NO];
        [zLbl setHidden:NO];
        [zTxt setHidden:NO];
    }
    else{
        [xViewBox setHidden:NO];
        [yViewBox setHidden:NO];
        [zViewBox setHidden:YES];
        [zLbl setHidden:YES];
        [zTxt setHidden:YES];
    }
    
    //more configuring of example...
    if(!strcmp(deviceSKU, "1043") || !strcmp(deviceSKU, "1044")){
        [xView setType:1];
        [yView setType:1];
        [zView setType:1];
        [accelLegend setHidden:NO];
    }
    else{
        [xView setType:0];
        [yView setType:0];
        [zView setType:0];
        [accelLegend setHidden:YES];
    }
    
    //Configure data interval slider with information read from channel
    [dataIntMinLbl setStringValue:[NSString stringWithFormat:@"%d",minDataInterval]];
    [dataIntMaxLbl setStringValue:[NSString stringWithFormat:@"%d",maxDataInterval]];
    [dataIntSlider setMaxValue:(double)maxDataInterval];
    [dataIntSlider setMinValue:(double)minDataInterval];
    [dataIntLbl setStringValue:[NSString stringWithFormat:@"%d ms",dataInterval]];
    [dataIntSlider setDoubleValue:(double)dataInterval];
    
    
    //Configure trigger slider with information read from channel
    [triggerMinLbl setStringValue:[NSString stringWithFormat:@"%0.1f",minAccelerationChangeTrigger]];
    [triggerMaxLbl setStringValue:[NSString stringWithFormat:@"%0.1f",maxAccelerationChangeTrigger]];
    [triggerSlider setMaxValue:maxAccelerationChangeTrigger];
    [triggerSlider setMinValue:minAccelerationChangeTrigger];
    [triggerLbl setStringValue:[NSString stringWithFormat:@"%0.3f g",accelerationChangeTrigger]];
    [triggerSlider setDoubleValue:accelerationChangeTrigger];
    
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    if([zViewBox isHidden]){
        frame.size.height = [mainWindow maxSize].height - zViewBox.frame.size.height;
    }
    else{
        frame.size.height = [mainWindow maxSize].height;
    }
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

- (void)onDetachHandler{
    //Taking items out of view
    [settingsBox setHidden:YES];
    [dataBox setHidden:YES];
    [xViewBox setHidden:YES];
    [yViewBox setHidden:YES];
    [zViewBox setHidden:YES];
    [phidgetInfoBoxView fillPhidgetInfo:nil];

    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

- (void)onAccelerometerChangeHandler:(NSArray *)chValues{
    //Update labels with new values
    double x = [[chValues objectAtIndex:0]doubleValue];
    double y = [[chValues objectAtIndex:1]doubleValue];
    double z = [[chValues objectAtIndex:2]doubleValue];
    [xTxt setStringValue:[NSString stringWithFormat:@"%.3f", x]];
    [yTxt setStringValue:[NSString stringWithFormat:@"%.3f", y]];
    [zTxt setStringValue:[NSString stringWithFormat:@"%.3f", z]];
    [timestamp setStringValue:[NSString stringWithFormat:@"%.0f",[[chValues objectAtIndex:3]doubleValue]]];
    
    //update gauges
    [xView setAngle:x];
    [yView setAngle:y];
    [zView setAngle:z];
}

static int errorCounter = 0;
-(void) outputError:(const char *)errorString{
    errorCounter++;
    NSAttributedString *outputString = [[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@\n",[NSString stringWithUTF8String:errorString]]
    	attributes:@{NSForegroundColorAttributeName: NSColor.controlTextColor}];
    [[errorEventLog textStorage] beginEditing];
    [[errorEventLog textStorage] appendAttributedString:outputString];
    [[errorEventLog textStorage] endEditing];
    
    [errorEventLogCounter setIntValue:errorCounter];
    if(![errorEventLogWindow isVisible])
        [errorEventLogWindow setIsVisible:YES];
}

-(void)errorHandler:(NSArray *)errorEventData{
    const char* errorString = [[errorEventData objectAtIndex:1] UTF8String];
    [self outputError:errorString];
}

-(void)addError:(PhidgetReturnCode)result{
    const char *errorString;
    Phidget_getErrorDescription(result, &errorString);
    [self outputError:errorString];
}

- (IBAction)clearErrorLog:(id)sender{
    [[errorEventLog textStorage] setAttributedString:[[NSAttributedString alloc] initWithString:@""]];
    [errorEventLogCounter setIntValue:0];
    errorCounter = 0;
}


#pragma mark GUI controls
- (IBAction)setChangeTrigger:(id)sender{
    PhidgetReturnCode result;
    NSEvent *currentEvent = [[sender window] currentEvent];
    if ([currentEvent type] == NSEventTypeLeftMouseUp) {
        result = PhidgetAccelerometer_setAccelerationChangeTrigger(ch, [triggerSlider doubleValue]);
        if(result != EPHIDGET_OK){
            [self addError:result];
            return;
        }
    }
    [triggerLbl setStringValue:[NSString stringWithFormat:@"%0.3f g",[triggerSlider doubleValue]]];
}

- (IBAction)setDataInterval:(id)sender{
    PhidgetReturnCode result;
    NSEvent *currentEvent = [[sender window] currentEvent];
    if ([currentEvent type] == NSEventTypeLeftMouseUp) {
        result = PhidgetAccelerometer_setDataInterval(ch, [dataIntSlider intValue]);
        if(result != EPHIDGET_OK){
            [self addError:result];
            return;
        }
    }
    [dataIntLbl setStringValue:[NSString stringWithFormat:@"%d ms",[dataIntSlider intValue]]];
}

@end
